@@ -16,18 +16,21 @@
 #include<assert.h>
 #include"types.c"
 #include"debug.e"
+#ifdef MISSING_STRDUP
+#define strdup(s)strcpy(malloc(strlen(s)+1),(s))
+#endif
 
 /*:2*/
 #line 44 "util.web"
 
 /*3:*/
-#line 71 "util.web"
+#line 74 "util.web"
 
-#line 72 "util.web"
+#line 75 "util.web"
 Class classes= NULL;
 
 /*:3*//*8:*/
-#line 160 "util.web"
+#line 163 "util.web"
 
 struct _STRING{
 char *s;
@@ -38,17 +41,17 @@ STRING hashtable[8009];
 int r2[8009],r1[8009];
 int nused= 0;
 
-/*:8*//*39:*/
-#line 737 "util.web"
+/*:8*//*41:*/
+#line 775 "util.web"
 
-#line 738 "util.web"
+#line 776 "util.web"
 Class toplevel= NULL;
 
-/*:39*/
+/*:41*/
 #line 45 "util.web"
 
 /*7:*/
-#line 125 "util.web"
+#line 128 "util.web"
 
 char *Strdup(s)
 char *s;
@@ -75,7 +78,7 @@ return t;
 }
 
 /*:7*//*9:*/
-#line 174 "util.web"
+#line 177 "util.web"
 
 static int hashval(name)
 char *name;
@@ -94,7 +97,7 @@ return h%8009;
 }
 
 /*:9*//*11:*/
-#line 207 "util.web"
+#line 210 "util.web"
 
 STRING hash(s)
 char *s;
@@ -126,7 +129,7 @@ return t;
 }
 
 /*:11*//*12:*/
-#line 245 "util.web"
+#line 248 "util.web"
 
 void delete(s)
 STRING s;
@@ -154,9 +157,9 @@ debug("linkcount on %s is %d\n",s->s,s->linkcount);
 }
 
 /*:12*//*13:*/
-#line 274 "util.web"
+#line 277 "util.web"
 
-#line 275 "util.web"
+#line 278 "util.web"
 char *get(s)
 STRING s;
 {
@@ -167,9 +170,9 @@ return s->s;
 }
 
 /*:13*//*14:*/
-#line 286 "util.web"
+#line 289 "util.web"
 
-#line 287 "util.web"
+#line 290 "util.web"
 STRING hdup(s)
 STRING s;
 {
@@ -179,9 +182,9 @@ return s;
 }
 
 /*:14*//*15:*/
-#line 298 "util.web"
+#line 301 "util.web"
 
-#line 299 "util.web"
+#line 302 "util.web"
 
 #ifdef USE_VARARGS
 STRING catstr(va_alist)va_dcl
@@ -236,9 +239,9 @@ return t;
 }
 
 /*:15*//*16:*/
-#line 355 "util.web"
+#line 358 "util.web"
 
-#line 356 "util.web"
+#line 359 "util.web"
 size_t Strlen(s)
 STRING s;
 {
@@ -249,9 +252,9 @@ return s!=((STRING)NULL)?(s->s!=NULL?strlen(s->s):0):0;
 #line 46 "util.web"
 
 /*17:*/
-#line 367 "util.web"
+#line 370 "util.web"
 
-#line 368 "util.web"
+#line 371 "util.web"
 STRING get_classname(s)
 STRING s;
 {
@@ -267,9 +270,9 @@ return t;
 }
 
 /*:17*//*18:*/
-#line 387 "util.web"
+#line 390 "util.web"
 
-#line 388 "util.web"
+#line 391 "util.web"
 STRING get_instname(s)
 STRING s;
 {
@@ -285,9 +288,9 @@ return t;
 }
 
 /*:18*//*19:*/
-#line 405 "util.web"
+#line 408 "util.web"
 
-#line 406 "util.web"
+#line 409 "util.web"
 STRING get_word(s)
 char *s;
 {
@@ -311,9 +314,9 @@ return t;
 #line 47 "util.web"
 
 /*4:*/
-#line 78 "util.web"
+#line 81 "util.web"
 
-#line 79 "util.web"
+#line 82 "util.web"
 void add_class(c)
 Class c;
 {
@@ -323,9 +326,9 @@ classes= c;
 }
 
 /*:4*//*5:*/
-#line 93 "util.web"
+#line 96 "util.web"
 
-#line 94 "util.web"
+#line 97 "util.web"
 Boolean get_option(value,c,name)
 STRING *value;
 Class c;
@@ -344,9 +347,9 @@ return False;
 }
 
 /*:5*//*20:*/
-#line 427 "util.web"
+#line 430 "util.web"
 
-#line 428 "util.web"
+#line 431 "util.web"
 STRING get_guard(c)
 Class c;
 {
@@ -357,9 +360,9 @@ return guard;
 }
 
 /*:20*//*21:*/
-#line 439 "util.web"
+#line 442 "util.web"
 
-#line 440 "util.web"
+#line 443 "util.web"
 STRING get_guardP(c)
 Class c;
 {
@@ -370,9 +373,9 @@ return guard;
 }
 
 /*:21*//*22:*/
-#line 455 "util.web"
+#line 458 "util.web"
 
-#line 456 "util.web"
+#line 459 "util.web"
 STRING get_headername(dir,c)
 char *dir;
 Class c;
@@ -383,16 +386,16 @@ char *s;
 if(file==((STRING)NULL))file= hash("file");
 s= Malloc((dir!=NULL?strlen(dir):0)+16);
 /*23:*/
-#line 473 "util.web"
+#line 476 "util.web"
 
-#line 474 "util.web"
+#line 477 "util.web"
 if(dir!=NULL&&dir[0]!='\0'){
 (void)strcpy(s,dir);
 if(dir[strlen(dir)-1]!='/')(void)strcat(s,"/");
 }
 
 /*:23*/
-#line 465 "util.web"
+#line 468 "util.web"
 
 if(!get_option(&t,c,file))t= c->name;
 (void)strncat(s,get(t),10);
@@ -402,9 +405,9 @@ return t;
 }
 
 /*:22*//*24:*/
-#line 484 "util.web"
+#line 487 "util.web"
 
-#line 485 "util.web"
+#line 488 "util.web"
 STRING get_headerPname(dir,c)
 char *dir;
 Class c;
@@ -415,16 +418,16 @@ STRING t;
 if(file==((STRING)NULL))file= hash("file");
 s= Malloc((dir!=NULL?strlen(dir):0)+16);
 /*23:*/
-#line 473 "util.web"
+#line 476 "util.web"
 
-#line 474 "util.web"
+#line 477 "util.web"
 if(dir!=NULL&&dir[0]!='\0'){
 (void)strcpy(s,dir);
 if(dir[strlen(dir)-1]!='/')(void)strcat(s,"/");
 }
 
 /*:23*/
-#line 494 "util.web"
+#line 497 "util.web"
 
 if(!get_option(&t,c,file))t= c->name;
 (void)strncat(s,get(t),9);
@@ -434,9 +437,9 @@ return t;
 }
 
 /*:24*//*25:*/
-#line 507 "util.web"
+#line 510 "util.web"
 
-#line 508 "util.web"
+#line 511 "util.web"
 STRING get_implementationname(dir,c)
 char *dir;
 Class c;
@@ -447,16 +450,16 @@ STRING t;
 if(file==((STRING)NULL))file= hash("file");
 s= Malloc((dir!=NULL?strlen(dir):0)+16);
 /*23:*/
-#line 473 "util.web"
+#line 476 "util.web"
 
-#line 474 "util.web"
+#line 477 "util.web"
 if(dir!=NULL&&dir[0]!='\0'){
 (void)strcpy(s,dir);
 if(dir[strlen(dir)-1]!='/')(void)strcat(s,"/");
 }
 
 /*:23*/
-#line 517 "util.web"
+#line 520 "util.web"
 
 if(!get_option(&t,c,file))t= c->name;
 (void)strncat(s,get(t),10);
@@ -466,9 +469,9 @@ return t;
 }
 
 /*:25*//*26:*/
-#line 528 "util.web"
+#line 531 "util.web"
 
-#line 529 "util.web"
+#line 532 "util.web"
 STRING get_docfilename(dir,c)
 char *dir;
 Class c;
@@ -479,16 +482,16 @@ STRING t;
 if(file==((STRING)NULL))file= hash("file");
 s= Malloc((dir!=NULL?strlen(dir):0)+16);
 /*23:*/
-#line 473 "util.web"
+#line 476 "util.web"
 
-#line 474 "util.web"
+#line 477 "util.web"
 if(dir!=NULL&&dir[0]!='\0'){
 (void)strcpy(s,dir);
 if(dir[strlen(dir)-1]!='/')(void)strcat(s,"/");
 }
 
 /*:23*/
-#line 538 "util.web"
+#line 541 "util.web"
 
 if(!get_option(&t,c,file))t= c->name;
 (void)strncat(s,get(t),8);
@@ -498,9 +501,9 @@ return t;
 }
 
 /*:26*//*28:*/
-#line 551 "util.web"
+#line 554 "util.web"
 
-#line 552 "util.web"
+#line 555 "util.web"
 static Class find_class(name)
 STRING name;
 {
@@ -512,9 +515,9 @@ return h;
 }
 
 /*:28*//*29:*/
-#line 565 "util.web"
+#line 568 "util.web"
 
-#line 566 "util.web"
+#line 569 "util.web"
 Class find_superclass(c,name)
 Class c;
 STRING name;
@@ -524,10 +527,10 @@ else if(c->name==name)return c;
 else return find_superclass(c->super,name);
 }
 
-/*:29*//*33:*/
-#line 630 "util.web"
+/*:29*//*34:*/
+#line 649 "util.web"
 
-#line 631 "util.web"
+#line 650 "util.web"
 Section find_method(c,m)
 Class c;
 STRING m;
@@ -541,10 +544,10 @@ if(d->decl&&d->decl->tp==Proc&&d->decl->name==m)return d;
 return NULL;
 }
 
-/*:33*//*34:*/
-#line 648 "util.web"
+/*:34*//*35:*/
+#line 667 "util.web"
 
-#line 649 "util.web"
+#line 668 "util.web"
 Boolean has_method(c,m)
 Class c;
 STRING m;
@@ -555,10 +558,10 @@ if(d->decl&&d->decl->tp==Proc&&d->decl->name==m)return True;
 return False;
 }
 
-/*:34*//*35:*/
-#line 662 "util.web"
+/*:35*//*36:*/
+#line 681 "util.web"
 
-#line 663 "util.web"
+#line 682 "util.web"
 Boolean has_classvar(c,m)
 Class c;
 STRING m;
@@ -569,10 +572,10 @@ if(d->decl&&d->decl->tp==Var&&d->decl->name==m)return True;
 return False;
 }
 
-/*:35*//*36:*/
-#line 676 "util.web"
+/*:36*//*37:*/
+#line 695 "util.web"
 
-#line 677 "util.web"
+#line 696 "util.web"
 Section find_pubvar(c,m)
 Class c;
 STRING m;
@@ -586,10 +589,29 @@ if(d->decl&&d->decl->tp==Var&&d->decl->name==m)return d;
 return NULL;
 }
 
-/*:36*//*37:*/
-#line 693 "util.web"
+/*:37*//*38:*/
+#line 712 "util.web"
 
-#line 694 "util.web"
+#line 713 "util.web"
+Section find_constraint(c,m)
+Class c;
+STRING m;
+{
+Section d,h;
+if(c==NULL)return NULL;
+if((h= find_constraint(c->super,m)))
+return h;
+for(d= c->constraints;d;d= d->next)
+if(d->decl&&d->decl->tp==Var&&d->decl->name==m)return d;
+for(d= c->privconstr;d;d= d->next)
+if(d->decl&&d->decl->tp==Var&&d->decl->name==m)return d;
+return NULL;
+}
+
+/*:38*//*39:*/
+#line 731 "util.web"
+
+#line 732 "util.web"
 STRING find_classvar_value(c,name)
 Class c;
 STRING name;
@@ -604,10 +626,10 @@ else
 return NULL;
 }
 
-/*:37*//*38:*/
-#line 711 "util.web"
+/*:39*//*40:*/
+#line 749 "util.web"
 
-#line 712 "util.web"
+#line 750 "util.web"
 STRING find_instvar_value(c,name)
 Class c;
 STRING name;
@@ -625,14 +647,15 @@ else
 return NULL;
 }
 
-/*:38*//*40:*/
-#line 745 "util.web"
+/*:40*//*42:*/
+#line 789 "util.web"
 
-#line 746 "util.web"
-void set_hierarchy()
+#line 790 "util.web"
+int set_hierarchy()
 {
-Class c;
+Class c,c1;
 char *p;
+int err= 0;
 for(c= classes;c;c= c->next){
 if((p= strrchr(get(c->superclass),'/')))
 c->super= find_class(hash(p+1));
@@ -646,15 +669,27 @@ c->sister= c->super->daughters;
 c->super->daughters= c;
 }
 }
+for(c= classes;c;c= c->next){
+for(c1= c->super;c1;c1= c1->super){
+if(c1==c){
+fprintf(stderr,
+"%s:%d: cyclic dependency found in superclasses of %s\n",
+get(c->filename),c->lineno,get(c->name));
+err++;
+break;
+}
+}
+}
+return err;
 }
 
-/*:40*/
+/*:42*/
 #line 48 "util.web"
 
 /*30:*/
-#line 581 "util.web"
+#line 585 "util.web"
 
-#line 582 "util.web"
+#line 586 "util.web"
 Class find_classvar_class(c,name)
 Class c;
 STRING name;
@@ -670,9 +705,9 @@ return NULL;
 }
 
 /*:30*//*31:*/
-#line 596 "util.web"
+#line 600 "util.web"
 
-#line 597 "util.web"
+#line 601 "util.web"
 Class find_instvar_class(c,name)
 Class c;
 STRING name;
@@ -689,9 +724,28 @@ return NULL;
 }
 
 /*:31*//*32:*/
-#line 612 "util.web"
+#line 616 "util.web"
 
-#line 613 "util.web"
+#line 617 "util.web"
+Class find_constr_class(c,name)
+Class c;
+STRING name;
+{
+Class h;
+Section d;
+if(c==NULL)return NULL;
+if((h= find_constr_class(c->super,name)))return h;
+for(d= c->constraints;d;d= d->next)
+if(d->decl&&d->decl->tp==Var&&d->decl->name==name)return c;
+for(d= c->privconstr;d;d= d->next)
+if(d->decl&&d->decl->tp==Var&&d->decl->name==name)return c;
+return NULL;
+}
+
+/*:32*//*33:*/
+#line 632 "util.web"
+
+#line 633 "util.web"
 Class find_method_class(c,name)
 Class c;
 STRING name;
@@ -699,20 +753,19 @@ STRING name;
 Class h;
 Section d;
 if(c==NULL)return NULL;
-if((h= find_method_class(c->super,name)))
-return h;
+if((h= find_method_class(c->super,name)))return h;
 for(d= c->methods;d;d= d->next)
 if(d->decl&&d->decl->tp==Proc&&d->decl->name==name)return c;
 return NULL;
 }
 
-/*:32*/
+/*:33*/
 #line 49 "util.web"
 
-/*41:*/
-#line 767 "util.web"
+/*43:*/
+#line 824 "util.web"
 
-#line 768 "util.web"
+#line 825 "util.web"
 Boolean strneq(a,b,n)
 char *a, *b;
 int n;
@@ -727,7 +780,7 @@ n--;
 return n==0|| *a== *b;
 }
 
-/*:41*/
+/*:43*/
 #line 50 "util.web"
 
 
